@@ -1,8 +1,8 @@
 package manifests
 
 import (
-	"fmt"
 	"path/filepath"
+	"sort"
 
 	"github.com/ghodss/yaml"
 	"github.com/pkg/errors"
@@ -16,16 +16,15 @@ import (
 	"github.com/openshift/installer/pkg/types/azure"
 	"github.com/openshift/installer/pkg/types/baremetal"
 	"github.com/openshift/installer/pkg/types/gcp"
+	"github.com/openshift/installer/pkg/types/kubevirt"
 	"github.com/openshift/installer/pkg/types/libvirt"
 	"github.com/openshift/installer/pkg/types/none"
 	"github.com/openshift/installer/pkg/types/openstack"
-	openstackdefaults "github.com/openshift/installer/pkg/types/openstack/defaults"
 	"github.com/openshift/installer/pkg/types/ovirt"
 	"github.com/openshift/installer/pkg/types/vsphere"
 )
 
 var (
-	infraCrdFilename           = filepath.Join(manifestDir, "cluster-infrastructure-01-crd.yaml")
 	infraCfgFilename           = filepath.Join(manifestDir, "cluster-infrastructure-02-config.yml")
 	cloudControllerUIDFilename = filepath.Join(manifestDir, "cloud-controller-uid-config.yml")
 )
@@ -70,41 +69,59 @@ func (i *Infrastructure) Generate(dependencies asset.Parents) error {
 			Name: "cluster",
 			// not namespaced
 		},
+		Spec: configv1.InfrastructureSpec{
+			PlatformSpec: configv1.PlatformSpec{},
+		},
 		Status: configv1.InfrastructureStatus{
 			InfrastructureName:   clusterID.InfraID,
 			APIServerURL:         getAPIServerURL(installConfig.Config),
 			APIServerInternalURL: getInternalAPIServerURL(installConfig.Config),
-			EtcdDiscoveryDomain:  getEtcdDiscoveryDomain(installConfig.Config),
 			PlatformStatus:       &configv1.PlatformStatus{},
 		},
 	}
 
 	switch installConfig.Config.Platform.Name() {
 	case aws.Name:
-		config.Status.PlatformStatus.Type = configv1.AWSPlatformType
+		config.Spec.PlatformSpec.Type = configv1.AWSPlatformType
+		config.Spec.PlatformSpec.AWS = &configv1.AWSPlatformSpec{}
 		config.Status.PlatformStatus.AWS = &configv1.AWSPlatformStatus{
 			Region: installConfig.Config.Platform.AWS.Region,
 		}
-	case azure.Name:
-		config.Status.PlatformStatus.Type = configv1.AzurePlatformType
 
-		rg := fmt.Sprintf("%s-rg", clusterID.InfraID)
+		for _, service := range installConfig.Config.Platform.AWS.ServiceEndpoints {
+			config.Spec.PlatformSpec.AWS.ServiceEndpoints = append(config.Spec.PlatformSpec.AWS.ServiceEndpoints, configv1.AWSServiceEndpoint{
+				Name: service.Name,
+				URL:  service.URL,
+			})
+			config.Status.PlatformStatus.AWS.ServiceEndpoints = append(config.Status.PlatformStatus.AWS.ServiceEndpoints, configv1.AWSServiceEndpoint{
+				Name: service.Name,
+				URL:  service.URL,
+			})
+			sort.Slice(config.Status.PlatformStatus.AWS.ServiceEndpoints, func(i, j int) bool {
+				return config.Status.PlatformStatus.AWS.ServiceEndpoints[i].Name <
+					config.Status.PlatformStatus.AWS.ServiceEndpoints[j].Name
+			})
+		}
+	case azure.Name:
+		config.Spec.PlatformSpec.Type = configv1.AzurePlatformType
+
+		rg := installConfig.Config.Azure.ClusterResourceGroupName(clusterID.InfraID)
 		config.Status.PlatformStatus.Azure = &configv1.AzurePlatformStatus{
 			ResourceGroupName:        rg,
 			NetworkResourceGroupName: rg,
+			CloudName:                configv1.AzureCloudEnvironment(installConfig.Config.Platform.Azure.CloudName),
 		}
 		if nrg := installConfig.Config.Platform.Azure.NetworkResourceGroupName; nrg != "" {
 			config.Status.PlatformStatus.Azure.NetworkResourceGroupName = nrg
 		}
 	case baremetal.Name:
-		config.Status.PlatformStatus.Type = configv1.BareMetalPlatformType
+		config.Spec.PlatformSpec.Type = configv1.BareMetalPlatformType
 		config.Status.PlatformStatus.BareMetal = &configv1.BareMetalPlatformStatus{
 			APIServerInternalIP: installConfig.Config.Platform.BareMetal.APIVIP,
-			NodeDNSIP:           installConfig.Config.Platform.BareMetal.DNSVIP,
 			IngressIP:           installConfig.Config.Platform.BareMetal.IngressVIP,
 		}
 	case gcp.Name:
-		config.Status.PlatformStatus.Type = configv1.GCPPlatformType
+		config.Spec.PlatformSpec.Type = configv1.GCPPlatformType
 		config.Status.PlatformStatus.GCP = &configv1.GCPPlatformStatus{
 			ProjectID: installConfig.Config.Platform.GCP.ProjectID,
 			Region:    installConfig.Config.Platform.GCP.Region,
@@ -119,48 +136,40 @@ func (i *Infrastructure) Generate(dependencies asset.Parents) error {
 			Data:     content,
 		})
 	case libvirt.Name:
-		config.Status.PlatformStatus.Type = configv1.LibvirtPlatformType
+		config.Spec.PlatformSpec.Type = configv1.LibvirtPlatformType
 	case none.Name:
-		config.Status.PlatformStatus.Type = configv1.NonePlatformType
+		config.Spec.PlatformSpec.Type = configv1.NonePlatformType
 	case openstack.Name:
-		config.Status.PlatformStatus.Type = configv1.OpenStackPlatformType
-		apiVIP, err := openstackdefaults.APIVIP(installConfig.Config.Networking)
-		if err != nil {
-			return err
-		}
-		dnsVIP, err := openstackdefaults.DNSVIP(installConfig.Config.Networking)
-		if err != nil {
-			return err
-		}
-		ingressVIP, err := openstackdefaults.IngressVIP(installConfig.Config.Networking)
-		if err != nil {
-			return err
-		}
+		config.Spec.PlatformSpec.Type = configv1.OpenStackPlatformType
 		config.Status.PlatformStatus.OpenStack = &configv1.OpenStackPlatformStatus{
-			APIServerInternalIP: apiVIP.String(),
-			NodeDNSIP:           dnsVIP.String(),
-			IngressIP:           ingressVIP.String(),
+			APIServerInternalIP: installConfig.Config.OpenStack.APIVIP,
+			IngressIP:           installConfig.Config.OpenStack.IngressVIP,
 		}
 	case vsphere.Name:
-		config.Status.PlatformStatus.Type = configv1.VSpherePlatformType
+		config.Spec.PlatformSpec.Type = configv1.VSpherePlatformType
 		if installConfig.Config.VSphere.APIVIP != "" {
 			config.Status.PlatformStatus.VSphere = &configv1.VSpherePlatformStatus{
 				APIServerInternalIP: installConfig.Config.VSphere.APIVIP,
-				NodeDNSIP:           installConfig.Config.VSphere.DNSVIP,
 				IngressIP:           installConfig.Config.VSphere.IngressVIP,
 			}
 		}
 	case ovirt.Name:
-		config.Status.PlatformStatus.Type = configv1.OvirtPlatformType
+		config.Spec.PlatformSpec.Type = configv1.OvirtPlatformType
 		config.Status.PlatformStatus.Ovirt = &configv1.OvirtPlatformStatus{
 			APIServerInternalIP: installConfig.Config.Ovirt.APIVIP,
-			NodeDNSIP:           installConfig.Config.Ovirt.DNSVIP,
 			IngressIP:           installConfig.Config.Ovirt.IngressVIP,
 		}
+	case kubevirt.Name:
+		config.Spec.PlatformSpec.Type = configv1.KubevirtPlatformType
+		config.Status.PlatformStatus.Kubevirt = &configv1.KubevirtPlatformStatus{
+			APIServerInternalIP: installConfig.Config.Kubevirt.APIVIP,
+			IngressIP:           installConfig.Config.Kubevirt.IngressVIP,
+		}
 	default:
-		config.Status.PlatformStatus.Type = configv1.NonePlatformType
+		config.Spec.PlatformSpec.Type = configv1.NonePlatformType
 	}
-	config.Status.Platform = config.Status.PlatformStatus.Type
+	config.Status.Platform = config.Spec.PlatformSpec.Type
+	config.Status.PlatformStatus.Type = config.Spec.PlatformSpec.Type
 
 	if cloudproviderconfig.ConfigMap != nil {
 		// set the configmap reference.
